<?php

/**
 * @file
 * AlertPay menu items.
 */

/**
 * Finalizes AlertPay transaction.
 */
function uc_alertpay_complete($cart_id = 0) {

  $site_security_code = variable_get('uc_alertpay_api_security_code', '');
  if ($site_security_code == '') {
    drupal_set_message(t('The administrator has not verified the security code.'));
    drupal_goto('');
  }

  $encrypted_data = $_REQUEST["AP"];
  $EPDDecryptor = new EPDDecryptor($site_security_code);
  $decrypted_data = $EPDDecryptor->decrypt($encrypted_data);

  watchdog('AlterPay', 'Receiving new order notification for order !order_id.', array('!order_id' => check_plain($decrypted_data["apc_1"])));
  if ($decrypted_data["ap_status"] != "Success") {
    return t('Paymet on AlertPay was not successfull.');
  }

  $order = uc_order_load($decrypted_data["apc_1"]);


  if ($order === FALSE || uc_order_status_data($order->order_status, 'state') != 'in_checkout') {
    return t('An error has occurred during payment.  Please contact us to ensure your order has submitted.');
  }
  // Empty that cart...
  uc_cart_empty($cart_id);

  // Add a comment to let sales team know this came in through the site.
  uc_order_comment_save($order->order_id, 0, t('Order created through website.'), 'admin');

  $build = uc_cart_complete_sale($order, variable_get('uc_new_customer_login', FALSE));

  $page = variable_get('uc_cart_checkout_complete_page', '');

  if (!empty($page)) {
    drupal_goto($page);
  }

  return $build;
}

/**
 *
 * EPDDecryptor
 *
 * A class which allows you to decrypt Payza's Encrypted Payment Details (EPD).
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY
 * OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @author Payza
 * @copyright 2010
 */

class EPDDecryptor {

    /**
     * The IPN security code that is generated from your Payza account
     * which is used as the key to decrypt the cypher text.
     */
  private $key;

    /**
     * One of the MCRYPT_ciphername constants of the name of the
   * algorithm as string
     */
  private $cipher;

    /**
     * One of the MCRYPT_MODE_modename constants
     */
  private $mode;

    /**
    * The iv parameter used for the initialisation in CB
    */
    private $iv;

    /**
    * A variable holding the parsed IPN variables
    */
  private $ipnData;

    /**
     * EPDDecryptor::__construct()
     *
     * Constructs a EPDDecryptor object and sets the
     * necessary variables
     *
     */
    public function __construct($securityCode) {
      $this->key = $securityCode;
      $this->iv = 'alertpay';
      $this->cipher = MCRYPT_3DES;
      $this->mode = MCRYPT_MODE_CBC;
    }

    /**
     * EPDDecryptor::decrypt()
     *
     * Decrypts the given text using the key provided.
     *
     * @param string $key The key used to decrypt.
     * @param string $data The encrypted text.
     *
     * @return string The decrypted text.
     */
  public function decrypt($cypherText) {
    //Decode the base64 encoded text
    $cypherText = base64_decode($cypherText);

        //Complete the key
    $key_add = 24-strlen($this->key);
    $this->key .= substr($this->key, 0, $key_add);

        // use mcrypt library for encryption
        $decryptedText = mcrypt_decrypt($this->cipher, $this->key, $cypherText, $this->mode, $this->iv);
        parse_str(trim($decryptedText, "\x00..\x1F"), $this->ipnData);

    return $this->ipnData;
  }
}

?>
